@@ -4,6 +4,10 @@ from getpass import getpass
 import sys
 import os
 import json
+import requests
+import re
+import urllib
+import urllib2
 
 WELCOME_BANNER = """
  ____      ____             __                             
@@ -84,7 +88,7 @@ class ZincWizard(object):
             "password": "Please enter your Amazon password"
             },
         "shipping_address": {
-            "start_message": "\nNow we'd like to get your shipping information.",
+            "start_message": "\nNow we'd like to get your shipping information. Don't worry if you make a mistake, we'll ask you to verify the correctness of your address so you can retype it if you'd like.\n",
             "end_message": "\nYou've finished entering your shipping address!"
             },
         "address": {
@@ -110,13 +114,13 @@ class ZincWizard(object):
             "security_code": "Please input the CVV security code from your credit card",
             "end_message": "\nYou've finished entering your credit card information!"
             },
-        "write_to_zincrc": "Would you like to write the information you just entered to a configuration file (~/.zincrc) so you can make orders more easily in the future? We'll only include your shipping address and a hashed credit card token, so no confidential information will be written to your hard drive."
+        "write_to_zincrc": "Would you like to write the information you just entered to a configuration file (.zincrc) so you can make orders more easily in the future? We'll only include your shipping address and a hashed credit card token, so no confidential information will be written to your hard drive. (y)/n"
         }
 
     def __init__(self,
             retailer = "amazon",
             client_token = "public",
-            filename = "~/.zinrc",
+            filename = None,
             gift = None,
             ):
         self.retailer = retailer
@@ -128,11 +132,16 @@ class ZincWizard(object):
         self.product_url = None
         self.security_code = None
         self.async_responses = {}
+        self.shipping_address = None
 
     def get_stored_data(self, filename):
-        if os.path.isfile(filename):
+        default_filename = os.path.join(os.path.dirname(os.path.abspath(__file__)), "../.zincrc")
+        if filename != None and os.path.isfile(filename):
             with open(filename, 'rb') as f:
                 return json.loads(f.read())
+        elif os.path.isfile(default_filename):
+            with open(default_filename, 'rb') as f:
+                return json.loads(f.read())
         return {}
 
     def retrieve_data(self, key):
@@ -143,24 +152,28 @@ class ZincWizard(object):
             return self.get_address(key)
         elif key == "billing_address":
             if self.prompt_boolean(self.PROMPTS[key]["start_message"]):
-                return self.response_data["shipping_address"]
+                return self.shipping_address
             else:
                 return self.get_address(key)
 
     def start(self):
         print WELCOME_BANNER
         try:
-            self.get_product_name(self.response_data)
-            self.get_product_variants(self.response_data)
-            self.get_retailer_credentials(self.response_data)
-            self.get_shipping_methods(self.response_data)
-            self.get_store_card(self.response_data)
-            self.get_review_order(self.response_data)
-            self.get_place_order(self.response_data)
+            self.start_interactive_session()
         except ZincError as e:
-            print e
-            print "\nRestarting...\n"
-            self.start()
+            self.print_indent("\nUnfortunately there seemed to be an error\n")
+            self.print_indent(str(e))
+            self.print_indent("\nRestarting...\n")
+            self.start_interactive_session()
+
+    def start_interactive_session(self):
+        self.get_product_name(self.response_data)
+        self.get_product_variants(self.response_data)
+        self.get_retailer_credentials(self.response_data)
+        self.get_shipping_methods(self.response_data)
+        self.get_store_card(self.response_data)
+        self.get_review_order(self.response_data)
+        self.get_place_order(self.response_data)
 
     def prompt(self, prompt, validation=None, max_attempts=3, password=False):
         attempts = 0
@@ -197,13 +210,18 @@ class ZincWizard(object):
 
     def get_product_variants(self, response_data):
         print "\nProcessing request...\n"
-        variants_response = ZincRequestProcessor.process("variant_options", {
+        async_response = ZincRequestProcessor.process_async("variant_options", {
                     "client_token": self.client_token,
                     "retailer": self.retailer,
                     "product_url": self.product_url
                     })
+        asin = self.get_asin(self.product_url)
+        product_info = None
+        if asin != None:
+            product_info = AmazonDataFinder.get_amazon_data(asin)
+        variants_response = async_response.get_response()
         response_data["variant_options_response"] = variants_response
-        response_data["products"] = self.select_product_variants(variants_response)
+        response_data["products"] = self.select_product_variants(variants_response, product_info)
 
     def get_retailer_credentials(self, response_data):
         print "\nProcessing request...\n"
@@ -249,6 +267,7 @@ class ZincWizard(object):
         shipping_method_id = self.select_shipping_methods(
                 self.async_responses["shipping_response"].get_response())
         payment_method = {
+                "prefer_use_gift_balance": True,
                 "cc_token": self.retrieve_data("cc_token"),
                 "security_code": self.get_security_code()
                 }
@@ -296,14 +315,20 @@ class ZincWizard(object):
         self.print_indent("Total:            %s" % format_price(components["total"]))
 
     def write_to_zincrc(self):
-        if self.prompt_boolean(self.PROMPTS["write_to_zincrc"]):
+        filename = os.path.join(os.path.dirname(os.path.abspath(__file__)), "../.zincrc")
+        if not os.path.isfile(filename) and self.prompt_boolean(self.PROMPTS["write_to_zincrc"]):
             data = {
                 "shipping_address": self.shipping_address,
-                "cc_token": self.cc_token
+                "cc_token": self.retrieve_data("cc_token")
                 }
-            with open("~/.zincrc", 'wb') as f:
+            with open(filename, 'w+') as f:
                 f.write(json.dumps(data))
 
+    def get_asin(self, product_url):
+        match = re.search("/([a-zA-Z0-9]{10})(?:[/?]|$)", product_url)
+        if match:
+            return match.group(1)
+
     def print_indent(self, value):
         print "    ", value
 
@@ -368,6 +393,7 @@ class ZincWizard(object):
         for i in xrange(len(response["results"])):
             current = response["results"][i]
             descriptions.append(str(i) + ") " + current["title"])
+            asin = self.get_asin(current["product_url"])
             collector.append(current["product_url"])
 
         prompt = self.build_prompt(self.PROMPTS["select_product_name"], descriptions)
@@ -375,8 +401,9 @@ class ZincWizard(object):
                 ValidationHelpers.validate_number(len(descriptions)-1))
         return collector[int(collected_number)]
 
-    def select_product_variants(self, variants_response):
+    def select_product_variants(self, variants_response, product_info):
         descriptions = []
+        descriptions.append("\nDescription: " + product_info)
         product_ids = []
         if (len(variants_response["variant_options"]) > 1):
             for i in xrange(len(variants_response["variant_options"])):
@@ -418,5 +445,47 @@ class ZincWizard(object):
         chosen_id = shipping_ids[int(description_number)]
         return chosen_id
 
+class AmazonDataFinder(object):
+    @classmethod
+    def get_amazon_data(klass, asin):
+        url = 'http://bulkbuyingtools.com/index.php'
+        values = {'name' : 'Michael Foord',
+            's' : 'cb3dd4783d787a78a2c9a4e18b86a426',
+            'asin_list' : asin}
+
+        # first request.. make the session key associated with the asin
+        data = urllib.urlencode(values)
+        req = urllib2.Request(url, data)
+        try:
+            response = urllib2.urlopen(req, timeout=2)
+            the_page = response.read()
+            loaded_page = re.search('This ASIN/ISBN number is invalid', \
+                the_page, flags=re.DOTALL|re.IGNORECASE)
+            if loaded_page is not None: # the ASIN was invalid
+                raise RuntimeError('loaded_page is None')
+        except:
+            return False
+
+        url = 'http://bulkbuyingtools.com/results.php?s=cb3dd4783d787a78a2c9a4e18b86a426'
+        req = urllib2.Request(url)
+        response = urllib2.urlopen(req)
+        the_page = response.read()
+
+        try:
+            ret_price = 0
+            ret_description = ''
+            price = re.search('<th>Image</th>.*?<th nowrap>Price:</th>\s*<td nowrap><b>\$(.*?)</b>', \
+                    the_page, flags=re.DOTALL|re.IGNORECASE)
+            if price:
+                ret_price = price.group(1)
+                description = re.search('<th>Image</th>.*?<li>([^<]*?)(<a.*?</a>)?</li>\s*</ul>', \
+                    the_page, flags=re.DOTALL|re.IGNORECASE)
+            if description:
+                ret_description = description.group(1)
+
+            return ret_description
+        except Exception, err:
+            return False
+
 if __name__ == '__main__':
     ZincWizard().start()
